/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/aggregate.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_aggregate
#define tadt_aggregate
/*
*   aggregate.h - calendric independent temporal support
*
*   This module contains aggregate functions for temporal types.
*   However, not all temporal types are defined for all of the aggregates.
*   Following is the functions; the type name follows the function indicating
*   that aggregate function can apply to the type.
*
*   	count: event	interval  span
*	sum:   			  span
*	avg:   event		  span
*	max:   event		  span
*	min:   event		  span
*
*/

/*  
*  Routine:  tadt_init_count_e
*
*  Description:  Allocate and initialize the integer used for the event 
*                count aggregate and return a pointer to it
*
*  Arguments:  None
*
*  Return Value: int *: allocated and initialized with 0 for computation
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
int *tadt_init_count_e();

/*  
*  Routine:  tadt_accum_count_e
*
*  Description: Accumulate the next event in the stream
*
*  Arguments:   count_state --  (IN&OUT) : intermediate count state
*		event           --  (IN) : the next event 
*  
*  Return Value: Error code
*
*  Errors: overflow
*
*  Side Effects: Overwrites count state
*/
error_type tadt_accum_count_e(int *count_state, event_type event);

/*  
*  Routine:  tadt_final_count_e
*
*  Description: Return the final count of events
*
*  Arguments: final_state -- (IN) : the final state contains the count
*             final_count -- (OUT): final count
*
*  Return Value: Error code
*  
*  Errors: None
*
*  Side Effects: Overwrites final_count
*/
error_type tadt_final_count_e(int final_state, int *final_count);

/*  
*  Routine:  tadt_init_avg_e
*
*  Description:  Allocate and initialize the structure for the function avg 
*                which computes the average of a column of events. 
*		 The struct contains a count of the number of events 
*		 and their sum, and both are initialized to 0.
*
*  Arguments:    None
*
*  Return Value: avg_state_e : allocated average event structure
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
avg_state_e tadt_init_avg_e();

/*  
*  Routine:  tadt_accum_avg_e
*
*  Description: Process the next event in the stream.
*		The new sum is calculated by adding the input event 
*		and the previously accumulated sum stored in 
*		avg_state. The sum in avg_state is then updated
*		with the new sum.
*
*  Arguments:   avg_state -- (IN&OUT) : intermediate average state
*		event         -- (IN) : the next event fed in
* 
*  Return Value:  Error code
* 
*  Errors: overflow
*
*  Side Effects: Overwrites avg_state
*/
error_type tadt_accum_avg_e(avg_state_e avg_state, event_type event);

/*  
*  Routine:  tadt_final_avg_e
*
*  Description: Return the final average of the events. The average is
*		computed from the division of the sum and the count which
*		were passed in the final_state (input argument).
*
*  Arguments: final_state -- (IN) : structure containing the total sum and count
*             event       -- (OUT): the final average, or sum / count
*  
*  Return Value:  Error code
*
*  Errors: divide_by_zero
*
*  Side Effects: Overwrites event
*/
error_type tadt_final_avg_e(avg_state_e final_state, event_type  event);

/*  
*  Routine:  tadt_init_max_e
*
*  Description:  Allocate and initialize the event for the function
*		 max which returns the maximum of a column of 
*                events in a relation. The event is initialized to a special
*                empty event.
*
*  Arguments:   None
*
*
*  Return Value: event : event used in computation of maximum event
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
event_type tadt_init_max_e();


/*  
*  Routine:  tadt_accum_max_e
*
*  Description: This function compares the input event with the current
*		maximum stored in max_state. If the input event
*		is larger than the maximum, then max_state is updated.
*
*  Arguments:   max_state --  (IN&OUT) : intermediate max state
*		event         --  (IN) : the next event fed in
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites max_state
*/
error_type tadt_accum_max_e(event_type max_state, event_type event);

/*  
*  Routine:  tadt_final_max_e
*
*  Description: Return the maximum event of a column 
*
*  Arguments: final_state --   (IN) : the final state contains the maximum event
*             event       --   (OUT): the result
*  
*  Return Value:  Error code
*
*  Errors: undefined_extremum
*
*  Side Effects: Overwrites event
*/
error_type tadt_final_max_e(event_type final_state, event_type event);

/*  
*  Routine:  tadt_init_min_e
*
*  Description:  Allocate and initialize the event for the function min 
*		 which returns the minimum of a column of events. 
*		 The event is initialized to a special empty event.
*
*  Arguments:  None
*
*  Return Value: event : event used in computation of maximum event
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
event_type tadt_init_min_e();

/*  
*  Routine:  tadt_accum_min_e
*
*  Description: This function compares the input event with the current 
*		minimum	stored in min_state. If the input event
*		is smaller than the minimum, then min_state is updated.
*
*  Arguments:   min_state -- (IN&OUT) : intermediate min state
*		event         -- (IN) : the new event fed in
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites min_state
*/
error_type tadt_accum_min_e(event_type min_state, event_type event);

/*  
*  Routine:  tadt_final_min_e
*
*  Description: Return the minimum event of a column 
*
*  Arguments: final_state -- (IN) : the final state contains the minimum event
*             event       -- (OUT): the result
*  
*  Return Value:  Error code
*
*  Errors: undefined_extremum
*
*  Side Effects: Overwrites event
*/
error_type tadt_final_min_e(event_type final_state, event_type event);

/*  
*  Routine:  tadt_init_count_i
*
*  Description:  Allocate and initialize the integer used for the function
*		 count which returns the maximum of a column of
*                intervals in a relation.  The count is initialized to 0.
*
*  Arguments:  None
*
*  Return Value: int *: pointer to allocated integer used to compute count
*
*  Errors: None
*
*  Side Effects: Allocates memory
*/
int *tadt_init_count_i();


/*  
*  Routine:  tadt_accum_count_i
*
*  Description: Process the next interval in the stream for the count aggregate
*
*  Arguments:   count_state --  (IN&OUT) : intermediate count state
*		interval    --  (IN)     : the next interval in the stream
*
*  Return Value:  Error code
*  
*  Errors: overflow
*
*  Side Effects: Overwrites count_state
*/
error_type tadt_accum_count_i(int *count_state, interval_type interval);

/*  
*  Routine:  tadt_final_count_i
*
*  Description: Return the final count of intervals 
*
*  Arguments: final_state -- (IN) : the final state contains the count
*             final_count -- (OUT): the final count
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites final_count
*/
error_type tadt_final_count_i(int final_state, int *final_count);

/*  
*  Routine:  tadt_init_count_fs
*
*  Description:  Allocate and initialize the integer used for the span count
*                aggregate which returns the count of a column of spans in
*                a relation.  The count is initialized to 0.
*
*  Arguments:  None
*
*
*  Return Value: int *: pointer to allocated integer used to compute count
*
*  Errors: None
*
*  Side Effects: Allocates memory
*/
int *tadt_init_count_fs();

/*  
*  Routine:  tadt_accum_count_fs
*
*  Description: Process the next span in the stream
*
*  Arguments:   count_state --  (IN&OUT) : intermediate count state
*		span            --  (IN) : the next span in the stream
*  
*  Return Value:  Error code
*
*  Errors: overflow
*
*  Side Effects: Overwrites count state
*/
error_type tadt_accum_count_fs(int *count_state, span_type span);

/*  
*  Routine:  tadt_final_count_fs
*
*  Description: Return the final count 
*
*  Arguments: final_state -- (IN) : the final state contains the count
*             final_count -- (OUT): the count
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites final_count
*/
error_type tadt_final_count_fs(int final_state, int *final_count);

/*  
*  Routine:  tadt_init_sum_fs
*
*  Description:  Allocate and initialize the span_type used for the function
*                sum.  The function computes the sum of a column of fixed
*		 spans.  The span is initialized to a 0 length span.
*
*  Arguments:    None
*
*  Return Value: span_type : structure used in computation of span sums
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
span_type tadt_init_sum_fs();

/*  
*  Routine:  tadt_accum_sum_fs
*
*  Description: The sum of the spans read so far. The sum is calculated
*		by adding the input span and the previously accumulated
*		sum stored in sum_state. sum_state is updated
*		with the new sum.
*		This function is called by the function sum providing an 
* 		argument of type span.
*
*  Arguments:   sum_state -- (IN&OUT) : intermediate sum state
*		span        -- (IN)     : the next span fed in
*  
*  Return Value:  Error code
*
*  Errors: overflow
*
*  Side Effects: Overwrites sum_state
*/
error_type tadt_accum_sum_fs(span_type sum_state, span_type span);

/*  
*  Routine:  tadt_final_sum_fs
*
*  Description: Return the total sum of the spans. The sum is actually
*		stored in the input, final_state, but this function is
*		needed for the original aggregate function sum to call.
*
*  Arguments: final_state --    (IN) : the final state
*             span    --    (OUT): the final result of sum of spans
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites span
*/
error_type tadt_final_sum_fs(span_type final_state, span_type span);

/*  
*  Routine:  tadt_init_avg_fs
*
*  Description:  Allocate and initialize the structure for the function avg which
*		 computes the average of a column of spans. 
*		 The struct contains a count for number of tuples
*		 and their sum.
*
*  Arguments:    None
*
*  Return Value: avg_state_fs : structure used in computation of avg of spans
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
avg_state_fs tadt_init_avg_fs();


/*  
*  Routine:  tadt_accum_avg_fs
*
*  Description: Accumulate the sum of the spans read so far. 
*		The new sum is calculated
*		by adding the input span and the previously accumulated sum
*		stored in avg_state. avg_sum is updated with the new sum.
*		This function is called by the function avg providing an 
* 		argument of type span.
*
*  Arguments:   avg_state --  (IN&OUT) : the previous state
*		span      --  (IN) : the next span in the stream
*  
*  Return Value:  Error code
*
*  Errors: overflow
*
*  Side Effects: Overwrites avg_state
*/
error_type tadt_accum_avg_fs(avg_state_fs avg_state, span_type span);

/*  
*  Routine:  tadt_final_avg_fs
*
*  Description: The final average of the spans. The average is
*		computed by dividing the total sum by the total count, 
*		stored in final_state.
*
*  Arguments: final_state -- (IN) : structure containing the total sum and count
*             span        -- (OUT): the final average of the spans
*  
*  Return Value:  Error code
*
*  Errors: divide_by_zero
*
*  Side Effects: Overwrites span
*/
error_type tadt_final_avg_fs(avg_state_fs final_state, span_type span);

/*  
*  Routine:  tadt_init_max_fs
*
*  Description:  Allocate and initialize the structure for the function max which
*		 returns the maximum of a column of spans.
*		 The struct contains a count for number of tuples 
*		 and the maximum span.
*
*  Arguments:    None
*
*  Return Value: span_type : used in the computation of the 
*                               maximum of the spans
*  
*  Errors: None
*
*  Side Effects: Allocates memory
*/
span_type tadt_init_max_fs();


/*  
*  Routine:  tadt_accum_max_fs
*
*  Description: This function compares the input span with the maximum
*		stored in max_state.  max_state is set to the
*		maximum of max_state and span.
*
*  Arguments:   max_state --  (IN&OUT) : structure containing the maximum of 
*				   previously read spans
*		span      --      (IN) : the next span in the stream
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites max_state
*/
error_type tadt_accum_max_fs(span_type max_state, span_type span);

/*  
*  Routine:  tadt_final_max_fs
*
*  Description: Return the maximal span of a column 
*
*  Arguments: final_state -- (IN) : the final state contains the maximum span
*             span        -- (OUT): the result
*  
*  Return Value:  Error code
*
*  Errors: undefined_extremum
*
*  Side Effects: Overwrites span
*/
error_type tadt_final_max_fs(span_type final_state, span_type span);

/*  
*  Routine:  tadt_init_min_fs
*
*  Description:  Allocate and initialize the structure for the function min which
*		 returns the minimum of a column of spans. 
*		 The struct contains a count for number of tuples 
*		 and the minimum span.
*
*  Arguments:    None
*
*  Return Value: span_type : used in computation of the minimum fixed span
*
*  Errors: None
*
*  Side Effects: Allocates memory
*/
span_type tadt_init_min_fs();

/*  
*  Routine:  tadt_accum_min_fs
*
*  Description: This function compares the input span with the minimum
*		stored in min_state. min_state is set to the 
*		minimum of min_state and span.
*
*  Arguments:   min_state --  (IN&OUT) : structure containing the minimum of 
*				        previously read spans
*		span          --  (IN) : the next span in the stream
*  
*  Return Value:  Error code
*
*  Errors: None
*
*  Side Effects: Overwrites min_state
*/
error_type tadt_accum_min_fs(span_type min_state, span_type span);

/*  
*  Routine:  tadt_final_min_fs
*
*  Description: Return the minimum span of a column 
*
*  Arguments: final_state -- (IN) : the final state contains the minimum span
*             span        -- (OUT): the result
*  
*  Return Value:  Error code
*
*  Errors: undefined_extremum
*
*  Side Effects: Overwrites span
*/
error_type tadt_final_min_fs(span_type final_state, span_type span);

#endif
