/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/arithmetic.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_arithmetic
#define tadt_arithmetic
/*  
*  Routine:  tadt_neg_fs
*
*  Description: Determine the sign opposite fixed span which is otherwise the
*		same as the input span.  Has no effect on a zero length
*               span.
*
*  Arguments: 	span -- (IN) : span to be converted
*               result -- (OUT) : result span 
*
*  Return Value: Error code
*
*  Errors: out_of_range (Max negative span has no positive counterpart) 
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_neg_fs(span_type span, span_type result);

/*  
*  Routine:  tadt_fs_add_fs
*
*  Description: Add two fixed spans
*
*  Arguments: 	span1, span2 -- (IN) : spans to be added
*               result -- (OUT) : result span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_fs_add_fs(span_type span1, span_type span2, span_type result);

/*  
*  Routine:  tadt_fs_minus_fs
*
*  Description: Subtract two fixed spans
*
*  Arguments: 	span1 -- (IN) : minuend (span to be subtracted from)
*		span2 -- (IN) : subtrahend
*               result -- (OUT) : result span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_fs_minus_fs(span_type span1, span_type span2, span_type result);

/*  
*  Routine:  tadt_e_add_fs
*
*  Description: Displace an event by adding a span 
*
*  Arguments: event -- (IN) : the event time
*	      span -- (IN) : the displacement
*             result -- (OUT) : result event
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_e_add_fs(event_type event, span_type span, event_type result);

/*  
*  Routine:  tadt_e_minus_fs
*
*  Description: Displace an event by subtracting a span 
*
*  Arguments: event -- (IN) : the event time
*	      span  -- (IN) : the displacement
*             result -- (OUT) : result event
*
*  Return Value: Error code
*
*  Errors: out_of_range
*          overflow  (beginning - anything)
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_e_minus_fs(event_type event, span_type span, event_type result);

/*  
*  Routine:  tadt_e_minus_e
*
*  Description: Compute the span between two events by 
*		subtracting the second event from the first event (e1 - e2).
*		if e1 > e2,  return a positive span
*		if e1 < e2,  return a negative span
*
*  Arguments:   event1, event2 -- (IN) : two input events.
*               result -- (OUT) : result span
*
*  Return Value: Error code
*
*  Errors: overflow  (forever - beginning)
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_e_minus_e(event_type event1, event_type event2, span_type result);

/*  
*  Routine:  tadt_fs_times_n
*
*  Description: Return a span equal to a multiple of the input span.  The     
*               multiplier must be an integer.
*
*  Arguments:   span -- (IN): the input span
*		n -- (IN): number of times
*               result -- (OUT) : result span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_fs_times_n(span_type span, int n, span_type result);

/*  
*  Routine:  tadt_fs_div_n
*
*  Description: Divide a span by a numeric value.  N must be nonzero.
*
*  Arguments: span -- (IN):the input span
*	      n -- (IN): number of division
*             result -- (OUT) : result span
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_fs_div_n(span_type span, int n, span_type result);

/*  
*  Routine:  tadt_fs_div_fs
*
*  Description: Divide one span by another
*
*  Arguments:   span1 -- (IN): the dividend
*	  	span2 --(IN): the divisor
*               result -- (OUT) : result integer
*
*  Return Value: Error code
*
*  Errors: out_of_range
*          division_by_zero
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_fs_div_fs(span_type span1, span_type span2, double *result);

/*  
*  Routine:  tadt_i_add_fs
*
*  Description: Displace an interval by adding a span
*
*  Arguments: interval -- (IN): the interval to be displaced
*	      span -- (IN): the span of displacement
*             result -- (OUT) : result interval
*
*  Return Value: Error code
*
*  Errors: out_of_range
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_i_add_fs(interval_type interval, span_type span, interval_type result);

/*  
*  Routine:  tadt_i_minus_fs
*
*  Description: Displace an interval by subtracting a span
*
*  Arguments: interval -- (IN): the interval to be displaced
*	      span -- (IN): the span of displacement
*             result -- (OUT) : result interval
*
*  Return Value: Error code
*
*  Errors: out_of_range
*          overflow     (|beginning,forever| - anything)
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_i_minus_fs(interval_type interval, span_type span, interval_type result);
#endif
