/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadt_timestamp.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 

/*  
*  Routine:  tadt_unswizzle_e  
*
*  Description:  Packs an event timestamp from the swizzled format.
*                The size of target event is passed in.  If the event 
*                is too big an error is generated.  An event packs
*                to whatever kind it started out as.
*
*  Arguments: event -- (IN): event structure
*             size  -- (IN): size of result 
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: tadt_not_enough_space
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unswizzle_e(event_type event, int size, char *result);

/*
*  Routine:  tadt_unswizzle_fs
*
*  Description:  Packs a span timestamp from the swizzled format.
*                The size of target span is passed in.  If the span
*                is too big an error is generated.  A span packs
*                to whatever kind it started out as.
*
*  Arguments: span -- (IN): span structure
*             size  -- (IN): size of result 
*             result -- (OUT): resulting string of characters
*
*  Return Value: Error code
*
*  Errors: tadt_not_enough_space
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unswizzle_fs(span_type span, int size, char *result);

/*
*  Routine:  tadt_unswizzle_i 
*
*  Description:  Packs an interval timestamp from the swizzled format.
*                The size of target interval is passed in.  If the interval
*                is too big an error is generated.  A interval packs
*                to whatever kind it started out as.
*
*  Arguments: interval -- (IN): interval structure
*             size  -- (IN): size of result 
*             result -- (OUT): resulting string of chars
*
*  Return Value: Error code
*
*  Errors: tadt_not_enough_space
*
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_unswizzle_i(interval_type interval, int size, char *result);

/*  
*  Routine:  tadt_swizzle_e
*
*  Description:  Swizzles a packed event timestamp into the event format.
*
*  Arguments: event -- (IN): event structure
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_e(packed_event_type event, event_type result);

/*  
*  Routine:  tadt_swizzle_fs
*
*  Description:  Swizzles a span timestamp into the unpacked format.
*
*  Arguments: span -- (IN): span structure
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_fs(packed_span_type span, span_type result);

/*
*  Routine:  tadt_swizzle_i 
*
*  Description:  Swizzles an interval timestamp into the unpacked format.
*
*  Arguments: interval -- (IN): interval structure
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: none
* 
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_swizzle_i(interval_type interval, interval_type result);

/*  
*  Routine:  tadt_create_indeterminate_e 
*
*  Description:  Builds an indeterminate event timestamp from the 
*                given information.  
*		 
*  Arguments: lower -- (IN): the lower support
*             upper -- (IN): the upper support
*             pmf -- (IN): the name of the probability mass function
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: tadt_bad_support_specified
*          tadt_inconsistent_granularity
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_indeterminate_e(event_type lower,
                                               event_type upper,
	                                       pmf_type pmf,
				               event_type result);
/*  
*  Routine:  tadt_create_indeterminate_now_relative_e 
*
*  Description:  Builds a indeterminate now-relative event timestamp from the 
*                given information.  
*		 
*  Arguments: limit -- (IN): the upper limit on the indeterminacy
*             relative -- (IN): a fixed or variable span that provides the 
*                               relative in now-relative
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: tadt_inconsistent_granularity
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_indeterminate_now_relative_e(event_type limit,
	                                            span_type relative,
				                    event_type result);
/*  
*  Routine:  tadt_create_now_relative_e  
*
*  Description:  Builds a now-relative event timestamp from the 
*                given information.  
*		 
*  Arguments: relative -- (IN): a fixed, variable, or indeterminate span 
*                               that provides the relative in now-relative
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_now_relative_e(span_type relative,
				      event_type result);

/*  
*  Routine:  tadt_create_special_e  
*
*  Description:  Builds a special event timestamp from the 
*                given information.  
*		 
*  Arguments: which -- (IN): which special guy is this?
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_special_e(int which, event_type result);

/*  
*  Routine:  tadt_create_indeterminate_fs 
*
*  Description:  Builds a indeterminate span timestamp from the 
*                given information.  
*		 
*  Arguments: lower -- (IN): the lower support
*             upper -- (IN): the upper support
*             pmf -- (IN): the name of the probability mass function
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: tadt_bad_support_specified
*          tadt_inconsistent_granularity
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_indeterminate_fs(span_type lower,
                                        span_type upper,
	                                pmf_type pmf,
				        span_type result);
/*  
*  Routine:  tadt_create_indeterminate_now_relative_fs 
*
*  Description:  Builds a indeterminate now-relative span timestamp from the 
*                given information.  
*		 
*  Arguments: limit -- (IN): the upper limit on the indeterminacy
*             relative -- (IN): a fixed or variable span that provides the 
*                               relative in now-relative
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: tadt_bad_suppport_specified
*          tadt_inconsistent_granularity
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_indeterminate_now_relative_fs(span_type limit,
	                                            span_type relative,
				                    span_type result);
/*  
*  Routine:  tadt_create_now_relative_fs 
*
*  Description:  Builds a now-relative span timestamp from the 
*                given information.  
*		 
*  Arguments: relative -- (IN): a fixed, variable, or indeterminate span 
*                               that provides the relative in now-relative
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_now_relative_fs(span_type relative,
				      span_type result);

/*  
*  Routine:  tadt_create_special_fs  
*
*  Description:  Builds a special span timestamp from the 
*                given information.  
*		 
*  Arguments: which -- (IN): which special guy is this?
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_special_fs(int which, span_type result);

/*  
*  Routine:  tadt_deconstruct_e 
*
*  Description:  Deconstructs an event
*		 
*  Arguments: event -- (IN): event to deconstruct
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_deconstruct_e(event_type event, deconstructed_event_type result);

/*  
*  Routine:  tadt_deconstruct_fs  
*
*  Description:  Deconstructs a fixed span
*		 
*  Arguments: span -- (IN): span to deconstruct
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_deconstruct_fs(span_type span, deconstructed_span_type result);

/*  
*  Routine:  tadt_deconstruct_i 
*
*  Description:  Deconstructs an interval into two deconstructed events
*		 
*  Arguments: interval -- (IN): interval to deconstruct
*             result_start -- (OUT): result event
*             result_finish -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_deconstruct_i(span_type span, deconstructed_event_type result_start, deconstructed_event_type result_finish);

/*  
*  Routine:  tadt_create_e 
*
*  Description:  Builds an event timestamp from the given chronon information.
*		 
*
*  Arguments: granularity -- (IN): granularity
*             size -- (IN): size of the thing
*             granules    -- (IN): number of granules
*             result -- (OUT): result event
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_e(granularity_type granularity, int size, polymorphic_int_type granules, event_type result);

/*  
*  Routine:  tadt_create_i
*
*  Description:  Builds an interval timestamp from the given events.  Assumes
*     storage for the interval has been pre-allocated.  Also tests the events
*     to make sure that it doesn`t build a bad interval.
*
*  Arguments: event1 -- (IN): starting event
*             event2 -- (IN): terminating event
*             result -- (OUT): result interval
*
*  Return Value: Error code
*
*  Errors: out_of_range
*          bad_interval
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_i(event_type event1, event_type event2, interval_type result);

/*  
*  Routine:  tadt_create_fs
*
*  Description:  Builds a fixed span from the given chronon information.
*
*  Arguments: granularity -- (IN): granularity
*             size -- (IN): size of the span to create
*             granules    -- (IN): number of granules
*             result -- (OUT): result span
*
*  Return Value: Error code
*
*  Errors: none
*  
*  Side Effects: Assignment to passed result structure
*/

error_type tadt_create_fs(granularity_type granularity, int size, polymorphic_int_type granules, span_type result);

